@@ -541,7 +541,9 @@ def evaluate_submission(
 
     # Collect snippets
     logging.log("Collecting snippets...")
-    snippet_markup = snippets.get_all_snippets(task_info)
+    # TODO: Better here
+    #snippet_markup = snippets.get_all_snippets(task_info)
+    snippet_markup = [ "For now, examples are not included in reports." ]
 
     # Now that we've got a rubric report, write it to the report file.
     logging.log(
