@@ -13,6 +13,7 @@ import pygments
 import pygments.lexers
 import pygments.formatters
 import importlib_resources
+import bs4
 
 from . import logging
 from . import html_tools
@@ -248,7 +249,8 @@ def render_instructions(
     rubric_table,
     snippets,
     output_file,
-    standalone=True
+    standalone=True,
+    report_rubric_link_coverage=False
 ):
     """
     Renders instructions in an HTML template and combines them with a
@@ -261,6 +263,10 @@ def render_instructions(
     The output file must be specified, and if you want to produce an HTML
     fragment (without CSS or JS) instead of a standalone HTML file you
     can pass standalone=False.
+
+    If report_rubric_link_coverage is set to True, messages about an
+    rubric goals which aren't linked to anywhere in the instructions will
+    be printed.
     """
     # Load our report template
     if standalone:
@@ -274,20 +280,100 @@ def render_instructions(
     # Augment our rubric table
     augment_report(rubric_table, blank=True)
 
-    # Render the HTML instructions to the html output file
-    with open(output_file, 'w', encoding="utf-8") as fout:
-        fout.write(
-            template.render(
-                taskid=task_info["id"],
-                task_info=task_info,
-                timestamp=timestamp,
-                instructions=instructions,
-                rendered_rubric=rubric_table["rendered_table"],
-                snippets=snippets,
-                css=get_css(),
-                js=get_js()
-            )
+    # Render the HTML instructions
+    rendered = template.render(
+        taskid=task_info["id"],
+        task_info=task_info,
+        timestamp=timestamp,
+        instructions=instructions,
+        rendered_rubric=rubric_table["rendered_table"],
+        snippets=snippets,
+        css=get_css(),
+        js=get_js()
+    )
+
+    # Check link coverage
+    if report_rubric_link_coverage:
+        # Find all anchor links
+        linked_to = set()
+        soup = bs4.BeautifulSoup(rendered, "html.parser")
+        for link in soup.find_all('a'):
+            ref = link.get('href')
+            if isinstance(ref, str) and ref.startswith('#goal:'):
+                linked_to.add(ref[1:])
+
+        # Find all goal IDs:
+        core_ids = set()
+        extra_ids = set()
+        other_ids = set()
+        soup = bs4.BeautifulSoup(
+            rubric_table["rendered_table"],
+            "html.parser"
         )
+        for node in soup.find_all(
+            lambda tag: (
+                tag.has_attr('id')
+            and tag.get('id').startswith('goal:')
+            )
+        ):
+            parent_classes = node.parent.get('class', [])
+            if 'tag-category-core' in parent_classes:
+                core_ids.add(node['id'])
+            elif 'tag-category-extra' in parent_classes:
+                extra_ids.add(node['id'])
+            else:
+                other_ids.add(node['id'])
+
+        core_unlinked = core_ids - linked_to
+        extra_unlinked = extra_ids - linked_to
+        other_unlinked = other_ids - linked_to
+
+        invalid_links = linked_to - (core_ids | extra_ids | other_ids)
+
+        total_unlinked = (
+            len(core_unlinked)
+          + len(extra_unlinked)
+          + len(other_unlinked)
+        )
+
+        if len(invalid_links) == 0:
+            print("All links to rubric goals were valid.")
+        else:
+            print(
+                (
+                    "{} link(s) to rubric goal(s) were invalid:"
+                ).format(len(invalid_links))
+            )
+            for inv in sorted(invalid_links):
+                print('  ' + inv)
+
+        if total_unlinked == 0:
+            print("All goals were linked to from the instructions.")
+        else:
+            print(
+                (
+                    "{} goal(s) were not linked to from the"
+                    " instructions:"
+                ).format(total_unlinked)
+            )
+            if len(core_unlinked) > 0:
+                print("Core goals that weren't linked:")
+                for goal in sorted(core_unlinked):
+                    print('  ' + goal)
+
+            if len(extra_unlinked) > 0:
+                print("Extra goals that weren't linked:")
+                for goal in sorted(extra_unlinked):
+                    print('  ' + goal)
+
+            if len(other_unlinked) > 0:
+                print("Other goals that weren't linked:")
+                for goal in sorted(other_unlinked):
+                    print('  ' + goal)
+
+    # Save to the output file
+    with open(output_file, 'w', encoding="utf-8") as fout:
+        fout.write(rendered)
 
 
 #-------------------#
